@@ -5,10 +5,15 @@ from collections import namedtuple
 from fontTools.misc.py23 import *
 from fontTools.misc import sstruct
 from fontTools import ttLib
+from fontTools import version
 from fontTools.misc.textTools import safeEval, pad
 from fontTools.misc.arrayTools import calcBounds, calcIntBounds, pointInRect
 from fontTools.misc.bezierTools import calcQuadraticBounds
-from fontTools.misc.fixedTools import fixedToFloat as fi2fl, floatToFixed as fl2fi
+from fontTools.misc.fixedTools import (
+	fixedToFloat as fi2fl,
+	floatToFixed as fl2fi,
+	otRound,
+)
 from numbers import Number
 from . import DefaultTable
 from . import ttProgram
@@ -16,10 +21,17 @@ import sys
 import struct
 import array
 import logging
-
+import os
+from fontTools.misc import xmlWriter
+from fontTools.misc.filenames import userNameToFileName
 
 log = logging.getLogger(__name__)
 
+# We compute the version the same as is computed in ttlib/__init__
+# so that we can write 'ttLibVersion' attribute of the glyf TTX files
+# when glyf is written to separate files.
+version = ".".join(version.split('.')[:2])
+
 #
 # The Apple and MS rasterizers behave differently for
 # scaled composite components: one does scale first and then translate
@@ -110,37 +122,64 @@ class table__g_l_y_f(DefaultTable.DefaultTable):
 			ttFont['maxp'].numGlyphs = len(self.glyphs)
 		return data
 
-	def toXML(self, writer, ttFont, progress=None):
-		writer.newline()
+	def toXML(self, writer, ttFont, splitGlyphs=False):
+		notice = (
+			"The xMin, yMin, xMax and yMax values\n"
+			"will be recalculated by the compiler.")
 		glyphNames = ttFont.getGlyphNames()
-		writer.comment("The xMin, yMin, xMax and yMax values\nwill be recalculated by the compiler.")
-		writer.newline()
-		writer.newline()
-		counter = 0
-		progressStep = 10
+		if not splitGlyphs:
+			writer.newline()
+			writer.comment(notice)
+			writer.newline()
+			writer.newline()
 		numGlyphs = len(glyphNames)
+		if splitGlyphs:
+			path, ext = os.path.splitext(writer.file.name)
+			existingGlyphFiles = set()
 		for glyphName in glyphNames:
-			if not counter % progressStep and progress is not None:
-				progress.setLabel("Dumping 'glyf' table... (%s)" % glyphName)
-				progress.increment(progressStep / numGlyphs)
-			counter = counter + 1
 			glyph = self[glyphName]
 			if glyph.numberOfContours:
-				writer.begintag('TTGlyph', [
-						("name", glyphName),
-						("xMin", glyph.xMin),
-						("yMin", glyph.yMin),
-						("xMax", glyph.xMax),
-						("yMax", glyph.yMax),
-						])
-				writer.newline()
-				glyph.toXML(writer, ttFont)
-				writer.endtag('TTGlyph')
-				writer.newline()
+				if splitGlyphs:
+					glyphPath = userNameToFileName(
+						tounicode(glyphName, 'utf-8'),
+						existingGlyphFiles,
+						prefix=path + ".",
+						suffix=ext)
+					existingGlyphFiles.add(glyphPath.lower())
+					glyphWriter = xmlWriter.XMLWriter(
+						glyphPath, idlefunc=writer.idlefunc,
+						newlinestr=writer.newlinestr)
+					glyphWriter.begintag("ttFont", ttLibVersion=version)
+					glyphWriter.newline()
+					glyphWriter.begintag("glyf")
+					glyphWriter.newline()
+					glyphWriter.comment(notice)
+					glyphWriter.newline()
+					writer.simpletag("TTGlyph", src=os.path.basename(glyphPath))
+				else:
+					glyphWriter = writer
+				glyphWriter.begintag('TTGlyph', [
+							("name", glyphName),
+							("xMin", glyph.xMin),
+							("yMin", glyph.yMin),
+							("xMax", glyph.xMax),
+							("yMax", glyph.yMax),
+							])
+				glyphWriter.newline()
+				glyph.toXML(glyphWriter, ttFont)
+				glyphWriter.endtag('TTGlyph')
+				glyphWriter.newline()
+				if splitGlyphs:
+					glyphWriter.endtag("glyf")
+					glyphWriter.newline()
+					glyphWriter.endtag("ttFont")
+					glyphWriter.newline()
+					glyphWriter.close()
 			else:
 				writer.simpletag('TTGlyph', name=glyphName)
 				writer.comment("contains no outline data")
-				writer.newline()
+				if not splitGlyphs:
+					writer.newline()
 			writer.newline()
 
 	def fromXML(self, name, attrs, content, ttFont):
@@ -174,6 +213,10 @@ class table__g_l_y_f(DefaultTable.DefaultTable):
 		# XXX optimize with reverse dict!!!
 		return self.glyphOrder.index(glyphName)
 
+	def removeHinting(self):
+		for glyph in self.glyphs.values():
+			glyph.removeHinting()
+
 	def keys(self):
 		return self.glyphs.keys()
 
@@ -217,8 +260,11 @@ flagYShort = 0x04
 flagRepeat = 0x08
 flagXsame =  0x10
 flagYsame = 0x20
-flagReserved1 = 0x40
-flagReserved2 = 0x80
+flagOverlapSimple = 0x40
+flagReserved = 0x80
+
+# These flags are kept for XML output after decompiling the coordinates
+keepFlags = flagOnCurve + flagOverlapSimple
 
 _flagSignBytes = {
 	0: 2,
@@ -365,10 +411,15 @@ class Glyph(object):
 				writer.begintag("contour")
 				writer.newline()
 				for j in range(last, self.endPtsOfContours[i] + 1):
-					writer.simpletag("pt", [
+					attrs = [
 							("x", self.coordinates[j][0]),
 							("y", self.coordinates[j][1]),
-							("on", self.flags[j] & flagOnCurve)])
+							("on", self.flags[j] & flagOnCurve),
+						]
+					if self.flags[j] & flagOverlapSimple:
+						# Apple's rasterizer uses flagOverlapSimple in the first contour/first pt to flag glyphs that contain overlapping contours
+						attrs.append(("overlap", 1))
+					writer.simpletag("pt", attrs)
 					writer.newline()
 				last = self.endPtsOfContours[i] + 1
 				writer.endtag("contour")
@@ -398,7 +449,10 @@ class Glyph(object):
 				if name != "pt":
 					continue  # ignore anything but "pt"
 				coordinates.append((safeEval(attrs["x"]), safeEval(attrs["y"])))
-				flags.append(not not safeEval(attrs["on"]))
+				flag = not not safeEval(attrs["on"])
+				if "overlap" in attrs and bool(safeEval(attrs["overlap"])):
+					flag |= flagOverlapSimple
+				flags.append(flag)
 			flags = array.array("B", flags)
 			if not hasattr(self, "coordinates"):
 				self.coordinates = coordinates
@@ -469,8 +523,7 @@ class Glyph(object):
 	def decompileCoordinates(self, data):
 		endPtsOfContours = array.array("h")
 		endPtsOfContours.fromstring(data[:2*self.numberOfContours])
-		if sys.byteorder != "big":
-			endPtsOfContours.byteswap()
+		if sys.byteorder != "big": endPtsOfContours.byteswap()
 		self.endPtsOfContours = endPtsOfContours.tolist()
 
 		data = data[2*self.numberOfContours:]
@@ -518,8 +571,8 @@ class Glyph(object):
 		assert xIndex == len(xCoordinates)
 		assert yIndex == len(yCoordinates)
 		coordinates.relativeToAbsolute()
-		# discard all flags but for "flagOnCurve"
-		self.flags = array.array("B", (f & flagOnCurve for f in flags))
+		# discard all flags except "keepFlags"
+		self.flags = array.array("B", (f & keepFlags for f in flags))
 
 	def decompileCoordinatesRaw(self, nCoordinates, data):
 		# unpack flags and prepare unpacking of coordinates
@@ -582,8 +635,7 @@ class Glyph(object):
 		assert len(self.coordinates) == len(self.flags)
 		data = []
 		endPtsOfContours = array.array("h", self.endPtsOfContours)
-		if sys.byteorder != "big":
-			endPtsOfContours.byteswap()
+		if sys.byteorder != "big": endPtsOfContours.byteswap()
 		data.append(endPtsOfContours.tostring())
 		instructions = self.program.getBytecode()
 		data.append(struct.pack(">h", len(instructions)))
@@ -789,7 +841,10 @@ class Glyph(object):
 			allEndPts = []
 			for compo in self.components:
 				g = glyfTable[compo.glyphName]
-				coordinates, endPts, flags = g.getCoordinates(glyfTable)
+				try:
+					coordinates, endPts, flags = g.getCoordinates(glyfTable)
+				except RecursionError:
+					raise ttLib.TTLibError("glyph '%s' contains a recursive component reference" % compo.glyphName)
 				if hasattr(compo, "firstPt"):
 					# move according to two reference points
 					x1,y1 = allCoords[compo.firstPt]
@@ -862,8 +917,12 @@ class Glyph(object):
 			expanding it."""
 		if not hasattr(self, "data"):
 			if remove_hinting:
-				self.program = ttProgram.Program()
-				self.program.fromBytecode([])
+				if self.isComposite():
+					if hasattr(self, "program"):
+						del self.program
+				else:
+					self.program = ttProgram.Program()
+					self.program.fromBytecode([])
 			# No padding to trim.
 			return
 		if not self.data:
@@ -985,6 +1044,39 @@ class Glyph(object):
 					cFlags = cFlags[nextOnCurve:]
 			pen.closePath()
 
+	def drawPoints(self, pen, glyfTable, offset=0):
+		"""Draw the glyph using the supplied pointPen. Opposed to Glyph.draw(),
+		this will not change the point indices.
+		"""
+
+		if self.isComposite():
+			for component in self.components:
+				glyphName, transform = component.getComponentInfo()
+				pen.addComponent(glyphName, transform)
+			return
+
+		coordinates, endPts, flags = self.getCoordinates(glyfTable)
+		if offset:
+			coordinates = coordinates.copy()
+			coordinates.translate((offset, 0))
+		start = 0
+		for end in endPts:
+			end = end + 1
+			contour = coordinates[start:end]
+			cFlags = flags[start:end]
+			start = end
+			pen.beginPath()
+			# Start with the appropriate segment type based on the final segment
+			segmentType = "line" if cFlags[-1] == 1 else "qcurve"
+			for i, pt in enumerate(contour):
+				if cFlags[i] == 1:
+					pen.addPoint(pt, segmentType=segmentType)
+					segmentType = "line"
+				else:
+					pen.addPoint(pt)
+					segmentType = "qcurve"
+			pen.endPath()
+
 	def __eq__(self, other):
 		if type(self) != type(other):
 			return NotImplemented
@@ -1053,7 +1145,7 @@ class GlyphComponent(object):
 		haveInstructions = self.flags & WE_HAVE_INSTRUCTIONS
 		self.flags = self.flags & (ROUND_XY_TO_GRID | USE_MY_METRICS |
 				SCALED_COMPONENT_OFFSET | UNSCALED_COMPONENT_OFFSET |
-				NON_OVERLAPPING)
+				NON_OVERLAPPING | OVERLAP_COMPOUND)
 		return more, haveInstructions, data
 
 	def compile(self, more, haveInstructions, glyfTable):
@@ -1062,7 +1154,7 @@ class GlyphComponent(object):
 		# reset all flags we will calculate ourselves
 		flags = self.flags & (ROUND_XY_TO_GRID | USE_MY_METRICS |
 				SCALED_COMPONENT_OFFSET | UNSCALED_COMPONENT_OFFSET |
-				NON_OVERLAPPING)
+				NON_OVERLAPPING | OVERLAP_COMPOUND)
 		if more:
 			flags = flags | MORE_COMPONENTS
 		if haveInstructions:
@@ -1075,8 +1167,8 @@ class GlyphComponent(object):
 				data = data + struct.pack(">HH", self.firstPt, self.secondPt)
 				flags = flags | ARG_1_AND_2_ARE_WORDS
 		else:
-			x = int(round(self.x))
-			y = int(round(self.y))
+			x = otRound(self.x)
+			y = otRound(self.y)
 			flags = flags | ARGS_ARE_XY_VALUES
 			if (-128 <= x <= 127) and (-128 <= y <= 127):
 				data = data + struct.pack(">bb", x, y)
@@ -1181,6 +1273,9 @@ class GlyphCoordinates(object):
 	def _checkFloat(self, p):
 		if self.isFloat():
 			return p
+		if any(v > 0x7FFF or v < -0x8000 for v in p):
+			self._ensureFloat()
+			return p
 		if any(isinstance(v, float) for v in p):
 			p = [int(v) if int(v) == v else v for v in p]
 			if any(isinstance(v, float) for v in p):
@@ -1220,7 +1315,6 @@ class GlyphCoordinates(object):
 		del self._a[i]
 		del self._a[i]
 
-
 	def __repr__(self):
 		return 'GlyphCoordinates(['+','.join(str(c) for c in self)+'])'
 
@@ -1238,15 +1332,16 @@ class GlyphCoordinates(object):
 			return
 		a = array.array("h")
 		for n in self._a:
-			a.append(int(round(n)))
+			a.append(otRound(n))
 		self._a = a
 
 	def relativeToAbsolute(self):
 		a = self._a
 		x,y = 0,0
 		for i in range(len(a) // 2):
-			a[2*i  ] = x = a[2*i  ] + x
-			a[2*i+1] = y = a[2*i+1] + y
+			x = a[2*i  ] + x
+			y = a[2*i+1] + y
+			self[i] = (x, y)
 
 	def absoluteToRelative(self):
 		a = self._a
@@ -1256,8 +1351,7 @@ class GlyphCoordinates(object):
 			dy = a[2*i+1] - y
 			x = a[2*i  ]
 			y = a[2*i+1]
-			a[2*i  ] = dx
-			a[2*i+1] = dy
+			self[i] = (dx, dy)
 
 	def translate(self, p):
 		"""
@@ -1266,8 +1360,7 @@ class GlyphCoordinates(object):
 		(x,y) = self._checkFloat(p)
 		a = self._a
 		for i in range(len(a) // 2):
-			a[2*i  ] += x
-			a[2*i+1] += y
+			self[i] = (a[2*i] + x, a[2*i+1] + y)
 
 	def scale(self, p):
 		"""
@@ -1276,8 +1369,7 @@ class GlyphCoordinates(object):
 		(x,y) = self._checkFloat(p)
 		a = self._a
 		for i in range(len(a) // 2):
-			a[2*i  ] *= x
-			a[2*i+1] *= y
+			self[i] = (a[2*i] * x, a[2*i+1] * y)
 
 	def transform(self, t):
 		"""
@@ -1393,8 +1485,8 @@ class GlyphCoordinates(object):
 			other = other._a
 			a = self._a
 			assert len(a) == len(other)
-			for i in range(len(a)):
-				a[i] += other[i]
+			for i in range(len(a) // 2):
+				self[i] = (a[2*i] + other[2*i], a[2*i+1] + other[2*i+1])
 			return self
 		return NotImplemented
 
@@ -1418,8 +1510,8 @@ class GlyphCoordinates(object):
 			other = other._a
 			a = self._a
 			assert len(a) == len(other)
-			for i in range(len(a)):
-				a[i] -= other[i]
+			for i in range(len(a) // 2):
+				self[i] = (a[2*i] - other[2*i], a[2*i+1] - other[2*i+1])
 			return self
 		return NotImplemented
 
